/*
 * gpiblib.c  -  GPIB interface library
 */

#include <windows.h>
#include <stdio.h>

#include "decl-32.h"
#include "gpiblib.h"

/** #define DEBUG **/

#define BOARD_NAME  "GPIB0"

int     GpibDevice; /* descriptor of the instrument */
int     GpibBoard;  /* descriptor of GPIB board */
int     GpibCount;  /* Saved value of ibcnt */
int     GpibStatus; /* Saved value of ibsta */

/* Open GPIB devices */
VOID
GpibOpen (device)
    char    *device;
{
    /* Assign unique id's to the device and the interface board.
       and check for errors. */
    if ((GpibDevice = ibfind (device)) < 0
        || (GpibBoard = ibfind (BOARD_NAME)) < 0)
    {
        GpibError ("ibfind Error: Unable to find device/board");
        exit (0);
    }

    /* Clear the device and check for errors. */
    if (ibclr (GpibDevice) < 0 || ibsre (GpibBoard, 0) < 0)
    {
        GpibError ("ibclr/ibsre Error: Unable to clear device/board");
        exit (0);
    }

    /* Set up the Event Status Enable Register, 
       and Service Request Enable Register to enable status events. */
    if (GpibWriteString ("*ESE 255") < 0
        || GpibWriteString ("*SRE 48") < 0)
    {
        GpibError ("GPIB write Error: Unable to initialize device");
        exit (0);
    }

    GpibTimeOut (T3s);
}

/* Close GPIB devices */
VOID
GpibClose ()
{
    GpibWriteString (":SYSTEM:KLOCK OFF");
}

/* Read a response from the device and wait for the read to finish. */
int
GpibRead (resp, cnt)
    char    *resp;
    int     cnt;
{
    /* Set the timeout to 10 seconds and wait for the response. */
    ibrd (GpibDevice, resp, cnt);
    GpibCount = ibcnt;
    GpibStatus = ibsta;
    resp[ibcnt] = '\0';
    return ibsta;
}

/* Send a string to the device and wait for the write to finish. */
int
GpibWriteString (string)
    char    *string;
{
    int     cnt = strlen(string);

#ifdef DEBUG
    fprintf (stderr, "GPIB Command: %s\n", string);
#endif

    /* Set the timeout to 10 seconds, send the string */
    ibwrt (GpibDevice, string, cnt);

    /* If ibwrt was successful, wait for completion. */
    if (ibsta >= 0)
        GpibWait (CMPL);

    GpibCount = ibcnt;
    GpibStatus = ibsta;
    return ibsta;
}

/* Send a binary data to the device and wait for the write to finish. */
int
GpibWrite (data, count)
    char    *data;
    long    count;
{
#ifdef DEBUG
    long    ii;

    fprintf (stderr, "GPIB Data: ");
    for (ii = 0; ii < count; ii++)
        fprintf (stderr, "%02x,", data[ii] & 0xff);
    fprintf (stderr, "\n");
#endif

    /* Set the timeout to 10 seconds, send the data */
    ibwrt (GpibDevice, data, count);

    /* If ibwrt was successful, wait for completion. */
    if (ibsta >= 0)
        GpibWait (CMPL);

    GpibCount = ibcnt;
    GpibStatus = ibsta;
    return ibsta;
}

/* Read a response from the device, and write it into a file. */
int
GpibReadFile (name)
    char    *name;
{
    return ibrdf (GpibDevice, name);
}

/* Read a string from a file, and write it to the device. */
int
GpibWriteFile (name)
    char    *name;
{
    return ibwrtf (GpibDevice, name);
}

/* Get status byte. */
int
GpibSerialPoll ()
{
    char    poll = 0;

    ibrsp (GpibDevice, &poll);
    return poll & 0xff;
}

/* Set time out. */
VOID
GpibTimeOut (tm)
    int     tm;
{
    ibtmo (GpibDevice, tm);
}

/* Wait. */
VOID
GpibWait (wt)
    int     wt;
{
    ibwait (GpibDevice, wt);
}

/* EOI control. */
VOID
GpibControlEOI (status)
    int     status;
{
    ibeot (GpibDevice, status);
}

/* Exit program with the device set in known state. */
VOID
GpibExit (code)
    int     code;
{
    GpibClose ();
    exit (code);
}

/* Display error from defined error codes based on the contents of ibsta. */
VOID
GpibError (msg)
    char    *msg;
{
    fprintf (stderr, "%s\n", msg);
    fprintf (stderr,  "ibsta=(%X)h  <", ibsta);

    if (ibsta & ERR ) fprintf (stderr, " ERR");
    if (ibsta & TIMO) fprintf (stderr, " TIMO");
    if (ibsta & END ) fprintf (stderr, " END");
    if (ibsta & SRQI) fprintf (stderr, " SRQI");
    if (ibsta & RQS ) fprintf (stderr, " RQS");
    if (ibsta & CMPL) fprintf (stderr, " CMPL");
    if (ibsta & LOK ) fprintf (stderr, " LOK");
    if (ibsta & REM ) fprintf (stderr, " REM");
    if (ibsta & CIC ) fprintf (stderr, " CIC");
    if (ibsta & ATN ) fprintf (stderr, " ATN");
    if (ibsta & TACS) fprintf (stderr, " TACS");
    if (ibsta & LACS) fprintf (stderr, " LACS");
    if (ibsta & DTAS) fprintf (stderr, " DTAS");
    if (ibsta & DCAS) fprintf (stderr, " DCAS");
    fprintf (stderr, " >\n");

    fprintf (stderr, "iberr= %d", iberr);
    if (iberr == EDVR) fprintf (stderr, " EDVR <DOS Error>\n");
    if (iberr == ECIC) fprintf (stderr, " ECIC <Not CIC>\n");
    if (iberr == ENOL) fprintf (stderr, " ENOL <No Listener>\n");
    if (iberr == EADR) fprintf (stderr, " EADR <Address error>\n");
    if (iberr == EARG) fprintf (stderr, " EARG <Invalid argument>\n");
    if (iberr == ESAC) fprintf (stderr, " ESAC <Not Sys Ctrlr>\n");
    if (iberr == EABO) fprintf (stderr, " EABO <Op. aborted>\n");

    if (iberr == ENEB) fprintf (stderr, " ENEB <No GPIB board>\n");
    if (iberr == EOIP) fprintf (stderr, " EOIP <Async I/O in prg>\n");
    if (iberr == ECAP) fprintf (stderr, " ECAP <No capability>\n");
    if (iberr == EFSO) fprintf (stderr, " EFSO <File sys. error>\n");
    if (iberr == EBUS) fprintf (stderr, " EBUS <Command error>\n");
    if (iberr == ESTB) fprintf (stderr, " ESTB <Status byte lost>\n");
    if (iberr == ESRQ) fprintf (stderr, " ESRQ <SRQ stuck on>\n");
/*  if (iberr == ETAB) fprintf (stderr, " ETAB <Table Overflow>\n"); */
}

/*
 * check error status of the GPIB instrument
 */

GpibCheckError ()
{
    int     errCode, first;
    char    buff[128 + 1];

    GpibWriteString ("*ESR?");
    GpibRead (buff, 128);
    errCode = atoi (buff) & 0x3c;  /* CME, EXE, DDE, QYE */
	do {
		strcpy(buff, "No error");
		if (errCode) {
			first = 1;
			GpibWriteString ("SYSTEM:ERROR?");
			do {
				GpibRead (buff, 128);
				buff[128] = '\0';
				if (strstr(buff, "No error") != NULL) break;
				if (first) {
					first = 0;
					fprintf (stderr, "\nGPIB Status: ");
				}
				fprintf (stderr, buff);
				fflush (stderr);
			} while (GpibCount == 128);
			/*fprintf (stderr, "\n");*/
		}
	} while (strstr(buff, "No error") == NULL);

    return errCode;
}
