/*
 *   putpat.c
 *
 *   a sample program to transfer the contents of the pattern file
 *   to the data storage of AWG.
 *
 *   Note: The return status of the GPIB library function is not
 *         fully checked in order to keep the code simple and
 *         understandable. You would be able to improve it
 *         with AWG local operations.
 *
 *   Copyright (C) Tektronix Japan, Ltd. All rights reserved.
 */

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "decl-32.h"
#include "..\gpiblib.h"

VOID ShowTitle();
VOID ShowUsage();
VOID MakeFilePathName();
char *FindTheLine();
int ReadPatDataHeader();
int ConvertByteData();
int ReadPatData();
VOID SendPatData();

#define TRUE    1
#define FALSE   0

long    DataLength; /* length of the pattern data */
float   ClockData;	/* pattern clock definition */
char    buffer[512];
char    FilePath[256];
char    FileName[41];
BOOL	is16bit = FALSE;


/*
 * the main routine
 */
VOID
main(argc, argv)
    int     argc;
    char    *argv[];
{
    FILE  *fp;        /* input pattern data file */
    short *patData;   /* pointer to allocated memory */
    char  *marker;    /* pointer to allocated memory */

    /* opening message */

    ShowTitle();

    /* check command line argument */

    if (argc != 3 && argc != 4)
    {
        ShowUsage();
        exit(1);
    }

	if (argc == 4)
	{
		if (strcmp(argv[1], "-16") == 0)
		{
			is16bit = TRUE;
			argv++;
		}
		else
		{
			ShowUsage();
			exit(1);
		}
	}
	MakeFilePathName(argv[2]);

    /* read header parameters of the word-oriented pattern file */

    if ((fp = fopen(argv[1], "r")) == NULL)
    {
        fprintf(stderr, "Error: Unable to open file '%s'\n", argv[1]);
        ShowUsage();
        exit(1);
    }

    if (! ReadPatDataHeader(fp))
    {
        fprintf(stderr, "Error: Unable to read file '%s'\n", argv[1]);
        exit(1);
    }

    fprintf(stderr, "sending data from file '%s'\n", argv[1]);
    fprintf(stderr, "             to   file '%s%s'\n", FilePath, FileName);

    /* initialize GPIB communication */

    GpibOpen("dev1");

    /* allocate memory to hold word-oriented pattern block */

	if (is16bit)
	{
		patData = malloc((size_t)(DataLength * 3L));
	}
	else
	{
		patData = malloc((size_t)(DataLength * 2L));
	}
    marker  = malloc((size_t)(DataLength));
    if (patData == NULL || marker == NULL)
    {
        fprintf(stderr, "Error: Unable to allocate work memory\n");
        GpibClose();
        exit(1);
    }

    /* read and send the pattern data block using work memory */

	fprintf(stderr, "CLOCK=%12.10e, SIZE=%ld\n", ClockData, DataLength);
    fprintf(stderr, "sending pattern data\n");
    ReadPatData(fp, patData, marker);
    SendPatData(patData, marker);

    GpibCheckError();

    /* close file, memory and GPIB communication */

    fclose(fp);
    free(patData);
    free(marker);
    GpibClose();
}

/*
 * show opening message
 */
VOID
ShowTitle()
{
    fprintf(stderr, "putpat - send the Pattern data\n");
    fprintf(stderr, "  Copyright (C) Tektronix Japan, Ltd. All rights reserved.\n\n");
}

/*
 * show command syntax
 */
VOID
ShowUsage()
{
    fprintf(stderr, "Usage: putpat [-16] <source-file> <destination-file>\n");
    fprintf(stderr, "  where <source-file> is the name of the pattern data file.\n");
    fprintf(stderr, "  where <destination-file> is the name of the AWG's mass storage file.\n");
	fprintf(stderr, "  Specify -16 if the target is 16 bits AWG, like AWG4XX.\n");
}

/*
 * make a file pass & filename from argument
 */
VOID
MakeFilePathName(file)
        char *file;
{
        char *p;

        p = strrchr(file, '/');
        if (p == NULL) {
                strcpy(FilePath, "./");
                strcpy(FileName, file);
        } else {
                strncpy(FilePath, file, strlen(file)-strlen(p)+1);
                strcpy(FileName, p+1);
        }
}

/*
 * find a line beginning with the specified keyword,
 * read the line into the buffer,
 * return the top of the successive token
 */

char *FindTheLine(fp, key)
    FILE    *fp;
    char    *key;
{
    char    *chp;
    int     keyLen = strlen(key);

    /* find the line */
    do {
        fgets(buffer, 128, fp);
        buffer[128] = '\0';
        if (feof(fp))
            return NULL;
    } while (strncmp(buffer, key, keyLen) != 0);

    /* find the second token */
    chp = &buffer[keyLen];
    while (*chp == ' ' || *chp == '\t')
        chp++;
    return chp;
}

/*
 * read header parameters of the word-oriented pattern file
 */
int
ReadPatDataHeader(fp)
    FILE    *fp;
{
    char    *chptr;

    chptr = FindTheLine(fp, "#CLOCK ");
    if (chptr == NULL)
        return FALSE;
    ClockData = (float)atof(chptr);

    chptr = FindTheLine(fp, "#SIZE ");
    if (chptr == NULL)
        return FALSE;
    DataLength = atol(chptr);

    return TRUE;
}

/*
 * read data block from the word-oriented pattern file
 */
int
ReadPatData(fp, patData, marker)
    FILE  *fp;
    short *patData;
	char  *marker;
{
    long    ll;
	short   m1, m2;

    /* read word data block */
    for (ll = 0; ll < DataLength; ll++)
    {
        fgets(buffer, 128, fp);
        if (feof(fp))
        {
            fprintf(stderr, "Error: Premature EOF\n");
            return FALSE;
        }

        buffer[128] = '\0';
        strupr(buffer);

		sscanf(buffer, "%hx,%hd,%hd,", &patData[ll], &m1, &m2);
		marker[ll] = (m1 == 0 ? 0 : 1) | (m2 == 0 ? 0 : 2);
    }
	return TRUE;
}

/*
 * send the word-oriented pattern data
 */
VOID
SendPatData(patData, marker)
    short *patData; /*  */
    char  *marker;
{
	long ll;
	short i;
    char work[16], *chr;
	char DataLengthStr[16];

    /* send command part and block header */

    GpibWriteString(":MMEMORY:MSIS \"MAIN\""); /* MassStorage is the Internal HD */
    sprintf(buffer, ":MMEMORY:CDIRECTORY \"%s\"", FilePath);
    GpibWriteString(buffer);

	if (is16bit)
	{
		/* data 2 bytes and marker 1byte */
		sprintf(DataLengthStr, "%ld", DataLength * 3L);
		sprintf(buffer, "#%d%s", strlen(DataLengthStr), DataLengthStr);
		i = strlen(buffer);
		sprintf(work, "%ld", 12 + i + DataLength * 3L + 25);
	}
	else
	{
		sprintf(DataLengthStr, "%ld", DataLength * 2L);
		sprintf(buffer, "#%d%s", strlen(DataLengthStr), DataLengthStr);
		i = strlen(buffer);
		sprintf(work, "%ld", 12 + i + DataLength * 2L + 25);
	}
	
				/* "MAGIC..." + "#..." + patData + "CLOCK..." */ 
    sprintf(buffer, ":MMEMORY:DATA \"%s\",#%d%s", FileName, strlen(work), work);
    GpibControlEOI(0); /* disable sending EOI */
    GpibWriteString(buffer); /* continued to write below */

    if (is16bit)
	{
		sprintf(buffer, "MAGIC 2003\r\n#%d%s", strlen(DataLengthStr), DataLengthStr);
		GpibWriteString(buffer); /* continued to write below */

		/* send arbitrary data block */
		for (ll = 0; ll < DataLength; ll++)
		{
			i = patData[ll];
			chr = (char *)(&i);
			buffer[0] = *chr;
			buffer[1] = *(chr+1);
			buffer[2] = (char)(marker[ll]);
			GpibWrite(buffer, 3);	/* continued to write below */
		}
	}
	else
	{
		sprintf(buffer, "MAGIC 2000\r\n#%d%s", strlen(DataLengthStr), DataLengthStr);
		GpibWriteString(buffer); /* continued to write below */

		/* send arbitrary data block */
		for (ll = 0; ll < DataLength; ll++)
		{
			i = patData[ll] | ((short)(marker[ll]) << 13);
			chr = (char *)(&i);
			buffer[0] = *chr;
			buffer[1] = *(chr+1);
			GpibWrite(buffer, 2);	/* continued to write below */
		}
	}

	sprintf(buffer, "CLOCK %12.10e\r\n", ClockData);
	GpibControlEOI(1); /* enable sending EOI */
    GpibWriteString(buffer);
}
